import 'dart:async';
import 'dmhub_sdk_method.dart';
import 'utils/dmhub_log.dart';
import 'models/dmhub_domain.dart';
import 'package:flutter/services.dart';

class DMHubClient {
  String _sdkVersion = '1.0.2';
  static const _channelPrefix = 'com.convertlab.dmhub';
  static const MethodChannel _channel =
      const MethodChannel('$_channelPrefix/dmhubsdk', JSONMethodCodec());
  static DMHubClient? _instance;
  static DMHubClient get getInstance =>
      _instance = _instance ?? DMHubClient._internal();

  DMHubOptions? _options;

  /// 获取配置信息[DMHubOptions].
  DMHubOptions? get options => _options;

  /// @nodoc private constructor
  DMHubClient._internal() {
    //
    DMHubLog.v('execute DMHubClient._internal');
  }

  /// 初始化SDK 指定[options]. 范围:android|ios
  Future<void> init(DMHubOptions options) async {
    _options = options;
    DMHubLog.v('-->init: $options');
    print(DMHubSDKMethod.init);
    Map result =
        await _channel.invokeMethod(DMHubSDKMethod.init, options.toJson());
    Map map = result[DMHubSDKMethod.init];
    DMHubLog.v('init->result:$map');
  }

  /// 初始化SDK 指定[options] 范围:ios
  Future<void> initWithOptions(DMHubOptions options) async {
    _options = options;
    DMHubLog.v('init: $options');

    Map result = await _channel.invokeMethod(
        DMHubSDKMethod.initWithOptions, options.toJson());
    Map map = result[DMHubSDKMethod.initWithOptions];

    DMHubLog.v('init result->$map');
  }

  /// 设置客户身份，用于绑定事件,如果重复设置，将会使用新的客户身份进行事件绑定
  ///
  /// @param identityType   身份类型
  ///
  /// @param identityValue  身份类型对应的值
  Future<void> setIdentity(String identityType, String identityValue) async {
    DMHubLog.v('setIdentity: $identityType : $identityValue');
    Map req = {'identityType': identityType, 'identityValue': identityValue};
    Map result = await _channel.invokeMethod(DMHubSDKMethod.setIdentity, req);
    // DMHubError.hasErrorFromResult(result);
    return result[DMHubSDKMethod.setIdentity];
  }

  /// 获取当前设置的客户身份
  ///
  /// @return 如果当前还未设置过有效的客户身份，返回 null
  Future<Map<String, dynamic>> currentIdentity() async {
    DMHubLog.v('currentIdentity method ');
    Map<String, dynamic> result =
        await _channel.invokeMethod(DMHubSDKMethod.currentIdentity, {});
    // DMHubError.hasErrorFromResult(result);
    return result;
  }

  /// 清除当前设置的客户身份
  ///
  /// @return 当前设置的客户身份，如果当前还未设置过有效的客户身份，返回 null
  Future<Map<String, dynamic>> clearIdentity() async {
    DMHubLog.v('clearIdentity method ');
    Map<String, dynamic> result =
        await _channel.invokeMethod(DMHubSDKMethod.clearIdentity, {});
    // DMHubError.hasErrorFromResult(result);
    return result;
  }

  /// 跟踪自定义客户事件
  ///
  /// @param eventId   在 DM Hub 中新建的自定义事件的事件 Id
  ///
  /// @param properties  事件的自定义属性，必须以在 DM Hub 中新建自定义事件时添加的自定义属性作为 key
  Future<void> trackMap(String eventId, Map<String, dynamic> properties) async {
    DMHubLog.v('trackMap: $eventId : $properties');
    Map req = {'eventId': eventId, 'properties': properties};
    Map result = await _channel.invokeMethod(DMHubSDKMethod.trackMap, req);
    // DMHubError.hasErrorFromResult(result);
    return result[DMHubSDKMethod.trackMap];
  }

  /// 跟踪自定义客户事件
  ///
  /// @param eventId   在 DM Hub 中新建的自定义事件的事件 Id
  ///
  Future<void> track(String eventId) async {
    DMHubLog.v('track->1 params: $eventId');
    Map req = {'eventId': eventId};
    Map result = await _channel.invokeMethod(DMHubSDKMethod.track, req);
    // DMHubError.hasErrorFromResult(result);
    return result[DMHubSDKMethod.track];
  }

  /// 跟踪进入视图事件
  ///
  /// @param viewName 视图的名称，客户时间轴上的显示为：'进入手机视图 ${viewName}'
  ///
  Future<void> trackOpenView(String viewName) async {
    DMHubLog.v('trackOpenView->viewName=$viewName');
    Map req = {'viewName': viewName};
    Map result = await _channel.invokeMethod(DMHubSDKMethod.trackOpenView, req);
    // DMHubError.hasErrorFromResult(result);
    return result[DMHubSDKMethod.trackOpenView];
  }

  /// 跟踪离开视图事件
  ///
  /// @param viewName 视图的名称，客户时间轴上的显示为：'离开手机视图 ${viewName}'
  ///
  Future<void> trackExitView(String viewName) async {
    DMHubLog.v('trackExitView->viewName=$viewName');
    Map req = {'viewName': viewName};
    Map result = await _channel.invokeMethod(DMHubSDKMethod.trackExitView, req);
    // DMHubError.hasErrorFromResult(result);
    return result[DMHubSDKMethod.trackExitView];
  }

  /// 设置消息推送的相关信息，以便通过 DM Hub 对客户进行 App 消息推送
  ///
  /// @param appKey 在推送服务商注册应用时获得的 AppKey
  ///
  /// @param provider 推送服务商，目前支持极光（@"jpush"）和个推（@"getui"）
  ///
  /// 使用范围: only ios
  Future<void> setAppKey(String appKey, String provider) async {
    DMHubLog.v('setAppKey->appKey=$appKey,provider=$provider');
    Map req = {'appKey': appKey, 'provider': provider};
    Map result = await _channel.invokeMethod(DMHubSDKMethod.trackExitView, req);
    // DMHubError.hasErrorFromResult(result);
    return result[DMHubSDKMethod.trackExitView];
  }

  /// 设置消息推送的相关信息，以便通过 DM Hub 对客户进行 App 消息推送
  ///
  /// @param appKey 在推送服务商注册应用时获得的 AppKey
  ///
  /// @param pushId 推送服务商分配给设备的标识，极光的 registrationID 或个推的 clientId
  ///
  /// @param provider 推送服务商，目前支持极光（@"jpush"）和个推（@"getui"）
  ///
  /// 使用范围: only ios
  Future<void> setAppKeyWithPushId(
      String appKey, String pushId, String provider) async {
    DMHubLog.v('setAppKey->appKey=$appKey,pushId=$pushId,provider=$provider');
    Map req = {'appKey': appKey, 'pushId': pushId, 'provider': provider};
    Map result = await _channel.invokeMethod(DMHubSDKMethod.trackExitView, req);
    // DMHubError.hasErrorFromResult(result);
    return result[DMHubSDKMethod.trackExitView];
  }

  /// 当自定义的 JPush 广播接收器收到来自 JPush 的广播时调用
  ///
  /// 使用范围: only android
  /// @decreapted
  Future<void> onJPushReceiveBroadcast() async {
    DMHubLog.v('onJPushReceiveBroadcast->');
    Map result =
        await _channel.invokeMethod(DMHubSDKMethod.onJPushReceiveBroadcast, {});
    // DMHubError.hasErrorFromResult(result);
    return result[DMHubSDKMethod.onJPushReceiveBroadcast];
  }

  /// JPush 注册
  ///
  /// @param appKey　用户申请的JPUSH　得到的appkey
  ///
  /// @param clientId 用户申请的JPUSH　得到的clientId
  ///
  /// 使用范围: android|ios
  ///
  Future<void> onJPushReceiveRegisterId(String appKey, String clientId) async {
    DMHubLog.v('onJPushReceiveRegisterId->: appKey=$appKey,clientId=$clientId');
    Map req = {'appKey': appKey, 'clientId': clientId};
    Map result = await _channel.invokeMethod(
        DMHubSDKMethod.onJPushReceiveRegisterId, req);
    // DMHubError.hasErrorFromResult(result);
    return result[DMHubSDKMethod.onJPushReceiveRegisterId];
  }

  /// JPush 数据接收
  ///
  /// @param bundle
  ///
  /// 使用范围: android|ios
  ///
  ///
  Future<void> onJPushReceiveMessageData(Map<String, dynamic> bundle) async {
    DMHubLog.v('onJPushReceiveRegisterId->: bundle=$bundle');
    Map result = await _channel.invokeMethod(
        DMHubSDKMethod.onJPushReceiveMessageData, bundle);
    // DMHubError.hasErrorFromResult(result);
    return result[DMHubSDKMethod.onJPushReceiveMessageData];
  }

  /// JPush 数据接收
  ///
  /// @param bundle
  ///
  /// 使用范围: android|ios
  ///
  ///
  Future<void> onJPushNotificationMessageArrived(
      Map<String, dynamic> bundle) async {
    DMHubLog.v('onJPushNotificationMessageArrived->: bundle=$bundle');
    Map result = await _channel.invokeMethod(
        DMHubSDKMethod.onJPushNotificationMessageArrived, bundle);
    // DMHubError.hasErrorFromResult(result);
    return result[DMHubSDKMethod.onJPushNotificationMessageArrived];
  }

  /// 在自定义的 GTIntentService 的 onReceiveClientId 方法中调用
  ///
  ///　@param appKey 收到的 appKey
  ///
  ///  @param clientId 收到的 clientId
  ///
  ///　使用范围: android|ios
  Future<void> onGeTuiReceiveClientId(String appKey, String clientId) async {
    DMHubLog.v('onGeTuiReceiveClientId->: appKey=$appKey,clientId=$clientId');
    Map req = {'appKey': appKey, 'clientId': clientId};
    Map result =
        await _channel.invokeMethod(DMHubSDKMethod.onGeTuiReceiveClientId, req);
    // DMHubError.hasErrorFromResult(result);
    return result[DMHubSDKMethod.onGeTuiReceiveClientId];
  }

  /// 在自定义的 GTIntentService 的 onReceiveMessageData 方法中调用
  ///
  ///@param taskId    GTTransmitMessage 中携带的 taskId
  ///
  ///@param messageId GTTransmitMessage 中携带的 messageId
  ///
  ///@param payload   GTTransmitMessage 中携带的 payload
  ///
  ///使用范围: only android
  Future<void> onGeTuiReceiveMessageData(
      String taskId, String messageId, Map<String, dynamic> playload) async {
    DMHubLog.v('onGeTuiReceiveMessageData->: $taskId,$messageId,$playload');
    Map req = {'taskId': taskId, 'messageId': messageId, 'playload': playload};
    Map result = await _channel.invokeMethod(
        DMHubSDKMethod.onGeTuiReceiveMessageData, req);
    // DMHubError.hasErrorFromResult(result);
    return result[DMHubSDKMethod.onGeTuiReceiveMessageData];
  }

  /// 在自定义的 GTIntentService 的 onNotificationMessageArrived 方法中调用
  ///
  ///@param taskId    GTTransmitMessage 中携带的 taskId
  ///
  ///@param messageId GTTransmitMessage 中携带的 messageId
  ///
  ///使用范围: only android
  Future<void> onGeTuiNotificationMessageArrived(
      String taskId, String messageId) async {
    DMHubLog.v('onGeTuiNotificationMessageArrived->: $taskId,$messageId');
    Map req = {'taskId': taskId, 'messageId': messageId};
    Map result = await _channel.invokeMethod(
        DMHubSDKMethod.onGeTuiNotificationMessageArrived, req);
    // DMHubError.hasErrorFromResult(result);
    return result[DMHubSDKMethod.onGeTuiNotificationMessageArrived];
  }

  /// 跟踪收到 JPush 自定义消息事件
  ///
  ///@param userInfo Flutter JPush回调onReceiveNotification方法中传入的参数
  ///
  ///使用范围: only ios
  Future<void> trackReceiveJPushMessage(Map<String, dynamic> userInfo) async {
    DMHubLog.v('trackReceiveJPushMessage->: $userInfo');
    Map result = await _channel.invokeMethod(
        DMHubSDKMethod.trackReceiveJPushMessage, userInfo);
    // DMHubError.hasErrorFromResult(result);
    return result[DMHubSDKMethod.trackReceiveJPushMessage];
  }

  /// 跟踪收到 GeTui 透传消息事件
  ///
  /// @param userInfo Flutter GeTui回调onReceivePayload方法中传入的参数
  ///
  ///使用范围: only ios
  Future<void> trackReceiveGeTuiPayloadData(
      Map<String, dynamic> userInfo) async {
    DMHubLog.v('trackReceiveGeTuiPayloadData->: $userInfo');
    Map result = await _channel.invokeMethod(
        DMHubSDKMethod.trackReceiveGeTuiPayloadData, userInfo);
    // DMHubError.hasErrorFromResult(result);
    return result[DMHubSDKMethod.trackReceiveGeTuiPayloadData];
  }

  /// 如果一个应用设置的追踪代码较多，而且他们之间有一些公共的参数，如果在每个追踪点都分别设置这些参数，会给代码维护带来不便。
  /// 为此，DMHubSDK引入了push方法，用来设置应用全局的参数。通过push方法设置的参数，在该应用后续的track方法调用时，会自动传递。
  ///
  ///@param req Map<String, Object> 中携带的 key/value
  ///
  ///使用范围: all
  Future<void> onGlobalParamsPush(Map<String, dynamic> req) async {
    DMHubLog.v('onGlobalParamsPush->: $req');
    Map result =
        await _channel.invokeMethod(DMHubSDKMethod.onGlobalParamsPush, req);
    // DMHubError.hasErrorFromResult(result);
    return result[DMHubSDKMethod.onGlobalParamsPush];
  }

  String get flutterSDKVersion => _sdkVersion;
}
